==> fastec_3.1.12_23.tar.xz: usr/local/include/fastec/fi_frame.h <==
/*!
 * @file fas_frame.h
 * @brief The Fastec Frame Image Class.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FASTEC_FRAME_H_
#define FASTEC_FRAME_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <stdint.h>
#include <memory>

#ifdef _WIN32
#include "gige_msg.h"
#else
#include <fastec/gige_msg.h>
#endif

#include "fas_video.h"
namespace FAS_VIDEO
{
//--------------------------------------------------------------------
// Constant Definitions
//--------------------------------------------------------------------
//! Planar overlay support.
const uint8_t MAX_PLANES = 4;

//! Create object conditions for status of FiFrame.
typedef enum FRAME_STATUS
{
    FRAME_NEW,              //!< FiFrame object is only constructed and not ready to be used.
    FRAME_READY,            //!< FiFrame object is prepared and ready for use.
    INVALID_FOURCC,         //!< FiFrame creation received unsupported FOUR_CC code.
    INVALID_ROW_ALIGN,      //!< FiFrame object could not validate image format.
    FRAME_ALLOC_ERR         //!< FiFrame object could not allocate the frame vector.
} FRAME_STATUS;

//! Create a planar data structure to provide basic planar data for an FiFrame.
typedef struct FRAME_PLANAR_DATA
{
    uint32_t planeCount;
    uint32_t rowsizePerPlane[MAX_PLANES];
    const uint8_t *planarData[MAX_PLANES];
} FRAME_PLANAR_DATA;

//! Define a generic vector array containing 8 bits of data to represent
//! the binary blob of frame data that can be attached and detached from
//! this FiFrame object.
typedef std::vector<uint8_t>    FRAME_DATA;

typedef enum FRAME_ORIENTATION
{
    TOP_DOWN,   // Frames are top-down (top of image is left-top)
    BOTTOM_UP,  // Frames are bottom-up (top of image is bottom-right)
} FRAME_ORIENTATION;

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class FiFrame
{
public:
    FiFrame();
    virtual ~FiFrame();

    // This object is non-copyable and non-copy assignable until work out how the
    // buffer management scheme plays out.
    FiFrame(const FiFrame& i) = delete;
    FiFrame& operator=(const FiFrame& i) = delete;

    // Factory to create a frame of data using externally managed data.
    static std::unique_ptr<FAS_VIDEO::FiFrame> CreateFrame(uint32_t frameFourCC,
                                                           uint32_t frameWidth,
                                                           uint32_t frameHeight,
                                                           uint32_t rowAlign,
                                                           FAS_VIDEO::FRAME_ORIENTATION frameOrientation = FAS_VIDEO::FRAME_ORIENTATION::TOP_DOWN,
                                                           const uint8_t *externalFrameDataPtrToUse = nullptr,
                                                           uint32_t externalFrameDataLenToUse = 0);

    //! @name Public Property Accessor Functions.
    //!
    //! These are the public functions used to access the properties of this object.
    //! @{

    virtual uint32_t GetWidth();
    virtual uint32_t GetHeight();
    virtual uint32_t GetRowAlign();
    virtual uint32_t GetFourCC();
    virtual uint32_t GetRowBufferSize();

    // Provides read-only access to frame data.  Only useful for
    // use while the life of this FiFrame object exists.
    virtual const uint8_t *GetFrameDataPtr();
    virtual uint32_t GetFrameDataSize();
    virtual bool GetPlanarData(FRAME_PLANAR_DATA& planarDataToSet);
    virtual FRAME_ORIENTATION GetFrameOrientation();

    //! @}

    virtual bool FrameReadyToReceiveData();
    virtual bool CopyFrameData(const uint8_t *frameDataToWrite, uint32_t frameDataSize, bool flipIt);

protected:

    virtual bool IsRowByteAlignValid(uint32_t rowByteAlignToCheck);
    virtual uint32_t GetFrameBufferSize();

    FAS_VIDEO::FRAME_STATUS frameStatus;

    uint32_t fourccType;            //!< Type of image data represented by this frame object.
    uint32_t frameWidth;            //!< Number of pixels in one row of frame data.
    uint32_t frameHeight;           //!< Number of image rows in a frame data.
    uint32_t frameBytesPerPixel;    //!< Number of 8-bit bytes used per pixel in the frame data.
    uint32_t rowAlign;              //!< Row alignment value.  Set to 1 if no alignment needed.
    uint32_t nPlanes;               //!< Number of planes that exist in this frame data.

    std::unique_ptr<FAS_VIDEO::FRAME_DATA> frameData;   //!< Vector array of frame data.
    FAS_VIDEO::FRAME_ORIENTATION frameOrientation;      //!< How is the frame data oriented.

    //! If this object was created with externalFrameData this
    //! member is set to the non-nullptr that was passed to
    //! CreateFrame.  The frameData will be nullptr in this case.
    const uint8_t *externalFrameDataPtr;

    //! If this object was created with externalFrameData this
    //! member is set to the length of externalFrameData which
    //! was passed to CreateFrame.
    uint32_t externalFrameDataLen;
};



} // End of FAS_VIDEO

#endif /* !FASTEC_FRAME_H_ */

==> fastec_3.4.9.tar.xz: usr/local/include/fastec/fi_frame.h <==
/*!
 * @file fas_frame.h
 * @brief The Fastec Frame Image Class.
 *
 * Copyright 2019 (c) Fastec Imaging as an unpublished work.
 * All Rights Reserved.
 *
 * The information contained herein is confidential property of
 * Fastec Imaging. The use, copying, transfer or disclosure of such
 * information is prohibited except by express written agreement
 * with Fastec Imaging.
 */

#ifndef FASTEC_FRAME_H_
#define FASTEC_FRAME_H_
#pragma once

//--------------------------------------------------------------------
// Includes
//--------------------------------------------------------------------

#include <stdint.h>
#include <memory>

#ifdef _WIN32
#include "gige_msg.h"
#else
#include <fastec/gige_msg.h>
#endif

#include "fas_video.h"
namespace FAS_VIDEO
{
//--------------------------------------------------------------------
// Constant Definitions
//--------------------------------------------------------------------
//! Planar overlay support.
const uint8_t MAX_PLANES = 4;

//! Create object conditions for status of FiFrame.
typedef enum FRAME_STATUS
{
    FRAME_NEW,              //!< FiFrame object is only constructed and not ready to be used.
    FRAME_READY,            //!< FiFrame object is prepared and ready for use.
    INVALID_FOURCC,         //!< FiFrame creation received unsupported FOUR_CC code.
    INVALID_ROW_ALIGN,      //!< FiFrame object could not validate image format.
    FRAME_ALLOC_ERR         //!< FiFrame object could not allocate the frame vector.
} FRAME_STATUS;

//! Create a planar data structure to provide basic planar data for an FiFrame.
typedef struct FRAME_PLANAR_DATA
{
    uint32_t planeCount;
    uint32_t rowsizePerPlane[MAX_PLANES];
    const uint8_t *planarData[MAX_PLANES];
} FRAME_PLANAR_DATA;

//! Define a generic vector array containing 8 bits of data to represent
//! the binary blob of frame data that can be attached and detached from
//! this FiFrame object.
typedef std::vector<uint8_t>    FRAME_DATA;

typedef enum FRAME_ORIENTATION
{
    TOP_DOWN,   // Frames are top-down (top of image is left-top)
    BOTTOM_UP,  // Frames are bottom-up (top of image is bottom-right)
} FRAME_ORIENTATION;

//--------------------------------------------------------------------
// Forward Declarations
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Class Declaration
//--------------------------------------------------------------------

class FiFrame
{
public:
    FiFrame();
    virtual ~FiFrame();

    // This object is non-copyable and non-copy assignable until work out how the
    // buffer management scheme plays out.
    FiFrame(const FiFrame& i) = delete;
    FiFrame& operator=(const FiFrame& i) = delete;

    // Factory to create a frame of data using externally managed data.
    static std::unique_ptr<FAS_VIDEO::FiFrame> CreateFrame(bool& frameCreatedOk,
                                                           uint32_t frameFourCC,
                                                           uint32_t frameWidth,
                                                           uint32_t frameHeight,
                                                           uint32_t rowAlign,
                                                           FAS_VIDEO::FRAME_ORIENTATION frameOrientation = FAS_VIDEO::FRAME_ORIENTATION::TOP_DOWN,
                                                           const uint8_t *externalFrameDataPtrToUse = nullptr,
                                                           uint32_t externalFrameDataLenToUse = 0);

    //! @name Public Property Accessor Functions.
    //!
    //! These are the public functions used to access the properties of this object.
    //! @{

    virtual uint32_t GetWidth();
    virtual uint32_t GetHeight();
    virtual uint32_t GetRowAlign();
    virtual uint32_t GetFourCC();
    virtual uint32_t GetRowBufferSize();

    // Provides read-only access to frame data.  Only useful for
    // use while the life of this FiFrame object exists.
    virtual const uint8_t *GetFrameDataPtr();
    virtual uint32_t GetFrameDataSize();
    virtual bool GetPlanarData(FRAME_PLANAR_DATA& planarDataToSet);
    virtual FRAME_ORIENTATION GetFrameOrientation();

    //! @}

    virtual bool FrameReadyToReceiveData();
    virtual bool CopyFrameData(const uint8_t *frameDataToWrite, uint32_t frameDataSize, bool flipIt);

protected:

    virtual bool IsRowByteAlignValid(uint32_t rowByteAlignToCheck);
    virtual uint32_t GetFrameBufferSize();

    FAS_VIDEO::FRAME_STATUS frameStatus;

    uint32_t fourccType;            //!< Type of image data represented by this frame object.
    uint32_t frameWidth;            //!< Number of pixels in one row of frame data.
    uint32_t frameHeight;           //!< Number of image rows in a frame data.
    uint32_t frameBytesPerPixel;    //!< Number of 8-bit bytes used per pixel in the frame data.
    uint32_t rowAlign;              //!< Row alignment value.  Set to 1 if no alignment needed.
    uint32_t nPlanes;               //!< Number of planes that exist in this frame data.

    std::unique_ptr<FAS_VIDEO::FRAME_DATA> frameData;   //!< Vector array of frame data.
    FAS_VIDEO::FRAME_ORIENTATION frameOrientation;      //!< How is the frame data oriented.

    //! If this object was created with externalFrameData this
    //! member is set to the non-nullptr that was passed to
    //! CreateFrame.  The frameData will be nullptr in this case.
    const uint8_t *externalFrameDataPtr;

    //! If this object was created with externalFrameData this
    //! member is set to the length of externalFrameData which
    //! was passed to CreateFrame.
    uint32_t externalFrameDataLen;
};



} // End of FAS_VIDEO

#endif /* !FASTEC_FRAME_H_ */
